@@ -33,6 +33,7 @@ class DocumentSession(object):
         self._defer_commands = set()
         self._number_of_requests_in_session = 0
         self._query = None
+        self.use_optimistic_concurrency = kwargs.get("use_optimistic_concurrency", False)
         self.advanced = Advanced(self)
 
     def __enter__(self):
@@ -198,9 +199,10 @@ class DocumentSession(object):
         self._known_missing_ids.add(self._documents_by_entity[entity]["key"])
         self._deleted_entities.add(entity)
 
-    def delete(self, key_or_entity):
+    def delete(self, key_or_entity, expected_change_vector=None):
         """
-        @param key_or_entity:can be the key or the entity we like to delete
+        @param str or object key_or_entity: Can be the key or the entity we like to delete
+        @param str expected_change_vector: A change vector to use for concurrency checks
         :type str or object:
         """
         if key_or_entity is None:
@@ -223,7 +225,7 @@ class DocumentSession(object):
             return
         self._known_missing_ids.add(key_or_entity)
         self._included_documents_by_id.pop(key_or_entity, None)
-        self._defer_commands.add(commands_data.DeleteCommandData(key_or_entity))
+        self._defer_commands.add(commands_data.DeleteCommandData(key_or_entity, expected_change_vector))
 
     def assert_no_non_unique_instance(self, entity, key):
         if not (key is None or key.endswith("/") or key not in self._documents_by_id
@@ -332,8 +334,8 @@ class DocumentSession(object):
             if key in self._documents_by_id:
                 existing_entity = self._documents_by_id[key]
                 if existing_entity in self._documents_by_entity:
-                    change_vector = self._documents_by_entity[existing_entity][
-                        "change_vector"] if self.advanced.use_optimistic_concurrency else None
+                    change_vector = self._documents_by_entity[existing_entity]["metadata"][
+                        "@change-vector"] if self.advanced.use_optimistic_concurrency else None
                 self._documents_by_entity.pop(existing_entity, None)
                 self._documents_by_id.pop(key, None)
             data.entities.append(existing_entity)
@@ -380,7 +382,8 @@ class DocumentSession(object):
 class Advanced(object):
     def __init__(self, session):
         self.session = session
-        self.use_optimistic_concurrency = False
+        self.use_optimistic_concurrency = session.use_optimistic_concurrency if session.use_optimistic_concurrency \
+            else session.conventions.use_optimistic_concurrency
 
     def number_of_requests_in_session(self):
         return self.session.number_of_requests_in_session
